#!/usr/bin/python3
#  coding: utf-8
#
#   setBeamPinNodes.py
#
#       指定されたNGRPの節点をpin構造に変更する。
#       （変位のみ伝達し、モーメントは伝達させない。）
#
#   20/01/23    新規作成
#

import sys
import geometryFunc as geo
import pyFistr

beamElms = ["611", "641"]


#
#  isBeamElements(elmType)
def isBeamElements(elmType):
    """ 要素Typeがbeamかどうかチェック"""
    if elmType in beamElms:
        return True
    else:
        return False

#
#  getOnlyBeamElements
def getOnlyBeamElements(nodeNos, nodeDict, elementDict):
    """ beam同士がつながっているnodeを取得する"""
    nodes = []
    for nodeNo in nodeNos:
        elms = nodeDict[nodeNo][1]
        counts = []
        for elmNo in elms:
            elmType = elementDict[elmNo][1]
            if isBeamElements(elmType) == True:
                counts.append(elmNo)
        if len(counts) >= 2:
            nodes.append(nodeNo)
    return nodes

#
#  getPinNodeNos
def getPinNodeNos(ngrps, meshHeaderData):
    """ pinNodeに変更するnodeNoを取得する。"""
    nodeNos = []
    for header, data in meshHeaderData:
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!NODE":
            ngrp = pyFistr.getValName(header, "NGRP")
            if ngrp in ngrps:
                nodes = list(map(lambda x: x[0], data))
                nodeNos += nodes
        elif words[0] == "!NGROUP":
            ngrp = pyFistr.getValName(header, "NGRP")
            if ngrp in ngrps:
                nodeNos += data
    return nodeNos

#
#  createNodeElementDict
def createNodeElementDict(meshHeaderData):
    """ nodeDict, elementDictを作成する"""
    nodeDict = {}
    elementDict = {}
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!NODE":
            data = headerData[1]
            for node in data:
                nodeNo = node[0]
                loc = node[1:]
                nodeDict[nodeNo] = [loc, []]
        elif words[0] == "!ELEMENT":
            data = headerData[1]
            elmType = pyFistr.getValName(header, "TYPE")
            for elm in data:
                elmNo = elm[0]
                nodes = elm[1:]
                elementDict[elmNo] = [nodes, elmType]
                for nodeNo in nodes:
                    nodeDict[nodeNo][1].append(elmNo)
    return nodeDict, elementDict

#
#  getPairNodesElements
def getPairNodesElements(nodeNos, nodeDict, elementDict):
    """ baseのnodeNoと同じ座標のnodeを追加する。"""
    pairNodeNos = []
    pairElmNos = []
    maxNodeNo = max(nodeDict.keys())
    for nodeNo in nodeNos:
        elms = nodeDict[nodeNo][1]
        beamElms = []
        for elmNo in elms:
            elmType = elementDict[elmNo][1]
            if isBeamElements(elmType) == True:
                beamElms.append(elmNo)
        pairElmNos.append(beamElms)
        pairNodes = [nodeNo]
        for i in range(1, len(beamElms)):
            maxNodeNo += 1
            newNodeNo = maxNodeNo
            pairNodes.append(newNodeNo)
        pairNodeNos.append(pairNodes)
    return pairNodeNos, pairElmNos

#
#  addNewNodes
def addNewNodes(pairNodes, meshHeaderData, nodeDict):
    """ newNodeを追加する"""
    header = "!NODE, NGRP=addedPinNodes\n"
    lines = []
    for pair in pairNodes:
        baseNodeNo = pair[0]
        newNodeNos = pair[1:]
        loc = nodeDict[baseNodeNo][0]
        for newNodeNo in newNodeNos:
            line = [newNodeNo] + loc
            lines.append(line)
    newHeaderData = [header, lines]
    #挿入場所を探す(!ELEMENTの前に挿入)
    for i in range(len(meshHeaderData)):
        headerData = meshHeaderData[i]
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!ELEMENT":
            si = i
            break
    meshHeaderData = meshHeaderData[:si] + [newHeaderData] + meshHeaderData[si:]
    return meshHeaderData

#
#  replaceNodeNo
def replaceNodeNo(pairNodes, pairElms, meshHeaderData):
    """ 要素内のnodeNoをnewNodeNoに置き換える"""

    def replaceElmNodeNo(elmNo, nodeNo, newNodeNo, meshHeaderData):
        """ elmNo内のnodeNoをnewNodeNoに置き換える"""
        flag = 0
        for i in range(len(meshHeaderData)):
            headerData = meshHeaderData[i]
            header = headerData[0]
            words = pyFistr.deleteSp(header).split(",")
            if words[0] == "!ELEMENT":
                elmType = pyFistr.getValName(header, "TYPE")
                if isBeamElements(elmType) == True:
                    data = headerData[1]
                    for ii in range(len(data)):
                        line = data[ii]
                        if line[0] == elmNo:
                            #置き換え
                            nodes = line[1:]
                            idx = nodes.index(nodeNo)
                            nodes[idx] = newNodeNo
                            data[ii] = [line[0]] + nodes
                            flag = 1
                            break
            if flag == 1:
                break
        return meshHeaderData

    for i in range(len(pairNodes)):
        baseNodeNo = pairNodes[i][0]
        newNodeNos = pairNodes[i][1:]
        baseElmNo = pairElms[i][0]
        remakeElmNos = pairElms[i][1:]
        for ii in range(len(remakeElmNos)):
            elmNo = remakeElmNos[ii]
            newNodeNo = newNodeNos[ii]
            meshHeaderData = replaceElmNodeNo(elmNo, baseNodeNo, newNodeNo, meshHeaderData)
    return meshHeaderData

#
#  createEquation
def createEquation(ngrps, pairNodes, pairElms, meshHeaderData, nodeDict, elementDict):
    """ equationを作成する"""

    def createEqLines(lineVals):
        newVals = []
        for i in range(0, len(lineVals), 3):
            coeff = lineVals[i+2]
            #係数が1.0e-8以下は、無視する。
            if abs(coeff) > 1.0e-8:
                newVals += lineVals[i:i+3]
        n = len(newVals) // 3
        line = str(n) + ", 0.0\n"
        line1 = ", ".join(list(map(str, newVals))) + "\n"
        return [line, line1]

    addHeaderData = []
    header = "#pinNodes " + ", ".join(ngrps) + "\n"
    addHeaderData += [[header, []]]
    header = "!EQUATION\n"
    lines = []
    for i in range(len(pairNodes)):
        nodeNo = pairNodes[i][0]
        for ii in range(1, len(pairNodes[i])):
            #変位を拘束
            newNodeNo = pairNodes[i][ii]
            lineData = [nodeNo, 1, 1, newNodeNo, 1, -1]
            addLines = createEqLines(lineData)
            lines += addLines
            lineData = [nodeNo, 2, 1, newNodeNo, 2, -1]
            addLines = createEqLines(lineData)
            lines += addLines
            lineData = [nodeNo, 3, 1, newNodeNo, 3, -1]
            addLines = createEqLines(lineData)
            lines += addLines
            #回転を拘束
            elmNo = pairElms[i][ii]
            nodes = elementDict[elmNo][0]
            node0 = nodeDict[nodes[0]][0]
            node1 = nodeDict[nodes[-1]][0]
            vec = geo.vector(node0, node1)
            vec = geo.normal(vec)
            lineData =  [nodeNo, 4, vec[0], nodeNo, 5, vec[1], nodeNo, 6, vec[2]]
            lineData += [newNodeNo, 4, -vec[0], newNodeNo, 5, -vec[1], newNodeNo, 6, -vec[2]]
            addLines = createEqLines(lineData)
            lines += addLines
    headerData = [header, lines]
    addHeaderData.append(headerData)
    #挿入場所は、!ENDの前
    si = -1
    for i in range(len(meshHeaderData)):
        header = meshHeaderData[i][0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!END":
            si = i
            break
    if si == -1:
        meshHeaderData = meshHeaderData + addHeaderData + [["!END", []]]
    else:
        meshHeaderData = meshHeaderData[:si] + addHeaderData + meshHeaderData[si:]
    return meshHeaderData

#-----------------
#  setPinNodes
#-----------------
def setPinNodes(ngrps):
    """ 指定されたNGRPの節点をpinNodeに変更する"""
    lines = pyFistr.readFistrModelMsh()
    meshHeaderData = pyFistr.getHeaderNumData(lines)
    nodeDict, elementDict = createNodeElementDict(meshHeaderData)
    nodeNos = getPinNodeNos(ngrps, meshHeaderData)
    nodeNos = getOnlyBeamElements(nodeNos, nodeDict, elementDict)
    nodeNos.sort()
    pairNodes, pairElms = getPairNodesElements(nodeNos, nodeDict, elementDict)
    meshHeaderData = addNewNodes(pairNodes, meshHeaderData, nodeDict)
    meshHeaderData = replaceNodeNo(pairNodes, pairElms, meshHeaderData)
    meshHeaderData = createEquation(ngrps, pairNodes, pairElms, meshHeaderData, nodeDict, elementDict)
    lines = pyFistr.createLinesFistrModelMshHeader(meshHeaderData)
    pyFistr.writeFistrModelMsh(lines)
    print("節点Grp「" + ", ".join(ngrps) + "」" + "をpinNodeに設定しました。")

#
#  printHelp
#------------
def printHelp():
    """ helpを出力"""
    cont  = "------------- setPinNodesOfBeam.py ---------------------\n"
    cont += "指定したNGRP名のnodeをpinNodeに置き換える。\n"
    cont += "（変位は伝達するがモーメントは伝達しないnode）\n"
    cont += "beam要素の611(6自由度)、641(3自由度)に対応。\n"
    cont += "<使い方>\n"
    cont += "  setPinNodesOfBeam.py <NRGP名1> <NGRP名2>...\n"
    cont += "<使用例>\n"
    cont += "  setPinNodesOfBeam.py linkNodes\n"
    cont += "--------------------------------------------------------\n"
    print(cont)

#
#  getArguments
#---------------
def getArguments():
    """ 引数を取得する"""
    if len(sys.argv) < 2:
        printHelp()
        return []
    ngrps = []
    for argv in sys.argv[1:]:
        if argv[0] == "-":
            printHelp()
            ngrps = []
            break
        else:
            ngrps.append(argv)
    return ngrps


if __name__ == "__main__":
    import gettext
    gettext.install("app")
    _ = gettext.gettext

    ngrps = getArguments()
    if len(ngrps) != 0:
        setPinNodes(ngrps)
