#!/usr/bin/python3
#  coding: utf-8
#
#       createVtkSourceFiles.py
#
#       meshのelement、surface、node group別にvtk形式で
#       fileを出力する。
#       出力先は、「vtkMeshData」folder内に出力。
#
#   20/06/15    新規作成
#      06/26    !EGROUPのvtkを追加
#      06/27    getVtkNodeData:offsetの値をi→i+1に修正
#               nodeGroupの最後のnodeが表示されない。
#               beamのnodeが表示されない。
#      06/30    path名表記の"/"をos.sepに修正
#      07/11    getSurfaceDataFromFaceNo:faceのnode順が2次要素の場合
#               間違っていたので、修正。
#      08/27    getVtkNodeData, getVtkSurfaceData, getVtkElementData:
#               座標値の出力精度を13.6e形式で出力する様に修正(精度を統一)
#               VTK表示で選択表示がうまく表示できない為、精度を統一               
#      09/02    createElementGrpVtkFiles:vtkFile作成がダブっていたので修正
#      09/04    changeSolidToFaceElement:追加
#               solidの場合、表面要素を抽出してvtkDataを作成する。
#               vtkFileの読み込みを早くする為。
#      09/8     allElement.vtuの作成を廃止。
#               要素Grpをoverlapして定義した場合、要素Grpを選択色に設定
#               すると、重ね書きした裏の要素色が見える事があるため、
#               各要素Grpのvtuファイル中に、overlapするしない事がわかる
#               CellDataを作成。
#               versionを設定
#      09/21    changeSolidToFaceElement:surElmDictにshellやbeamを追加。
#               9/8の修正で、shell、beamでエラー発生した為。
#      09/27    nodeGrpのデータをpointDataに追加。
#      09/28    surfaceGrp, nodeGrpの情報ををelementGrp内に挿入したので
#               これらのvtkFileの作成を止める。
#               vtkFileのversionを「1.2」に更新
#      09/30    clearSaveFolder:rmtreeで削除されるまで待つ事を追加
#               createNodeGrpVtkFiles:ngrpを!NODEからも取得する様に修正
#               dymmy節点grpが取得できなかった為。
#      10/01    changeSolidToFaceElement:elmNoとfaceNoの取得を追加
#               getVtkElementData:elmNoとfaceNoの保存を追加
#      10/09    createNodeGrpNameDict,getVtkElementData:NGRPがoverlapして
#               定義されていた場合、lapした部分が取得できていなかったので
#               修正。
#      10/10    addNodeGrp, addSurfacegrp:外からモジュール関数として使う為
#               に追加。
#      10/16    renameNodeGrp, renameSurfaceGrp, renameElementGrp,
#               deleteNodeGrp, deleteSurfaceGrp, deleteElementGrpを追加。
#               外からモジュール関数として使用する為。
#      10/21    addDeleteNodeGrp, addDeleteSurfaceGrp, addDeleteElementGrp
#               を追加。外からモジュール関数として使用する為。
#      10/25    edgeElementGrpのedgeVtkファイルを追加
#   21/01/06    createVtkFiles:モジュール関数として使える様に修正
#      01/14    createElementGrpVtkFiles:!ELEMENTでEGRPが未定義の場合
#               「--undefine--」名でvtkファイルを作成する様に修正
#      01/17    createElementGrpVtkFiles:バグ修正（vtkデータが正しくない）
#               辞書を作りながらvtk作成 → 辞書を作成後、vtk作成 に変更
#      04/05    shell,beamでもelmNoを書き込むように修正。（ver 1.33）
#      04/09    changeSolidToFaceElement:faceNoでsolid, shell, beamが区別
#               できる様に修正
#

import os
import sys
import shutil
import time
import glob
import multiprocessing as mp

import pyFistr
import convertMesh as cm


version = "1.34"                    #vtkFileのformat確認用
currDir = ""
SaveFolder = "vtkMeshData"          #vtkFileを保存するfolder名

surElmDict = {}
nodeGrpNameDict = {}

#
#  readFileLines
#----------------
def readFileLines(fileName):
    f=open(fileName, encoding="utf-8"); lines=f.readlines(); f.close()
    return lines

#
#  writeFileLines
#-----------------
def writeFileLines(fileName, lines):
    f=open(fileName, "w", encoding="utf-8"); f.writelines(lines); f.close()
    return

#
#  getMeshHeaderData
#---------------------
def getMeshHeaderData():
    """ meshDataを取得"""
    lines = pyFistr.readFistrModelMsh()
    meshHeaderData = pyFistr.getHeaderNumData(lines)
    return meshHeaderData

#
#  getNodesDict
#---------------
def getNodesDict(meshHeaderData):
    """ 節点Noをkeyとして、座標を求めるnodesDictを作成する。"""
    nodesDict = {}
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!NODE":
            data = headerData[1]
            for nodeData in data:
                nodeNo = nodeData[0]
                loc = nodeData[1:]
                nodesDict[nodeNo] = loc
    return nodesDict

#
#  getElementDict
#------------------
def getElementDict(meshHeaderData):
    """ 要素Noをkeyとして、節点列・要素typeが取得できる
    elementDictを作成する。"""
    elementDict = {}
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!ELEMENT":
            elType = pyFistr.getValName(header, "TYPE")
            data = headerData[1]
            for elmData in data:
                elNo = elmData[0]
                nodes = elmData[1:]
                elementDict[elNo] = [nodes, elType]
    return elementDict

#
#  clearSaveFolder
#------------------
def clearSaveFolder():
    """ folder内をクリアする"""
    #folderDir = currDir + "/" + SaveFolder
    folderDir = currDir + os.sep + SaveFolder
    if os.path.exists(folderDir) == True:
        shutil.rmtree(folderDir)
        #削除されるまで待つ
        while os.path.exists(folderDir) == True:
            time.sleep(0.1)
        os.mkdir(folderDir)
    else:
        os.mkdir(folderDir)

#
#  createNodeGrpVtkFiles
#-----------------------
def createNodeGrpVtkFiles(meshHeaderData, nodesDict):
    """ 節点group毎のvtkFileを作成する。"""
    #nodeGroupを取得
    nodeGrps = []
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!NGROUP":
            name = pyFistr.getValName(header, "NGRP")
            data = headerData[1]
            nodeGrps.append([name, data])
        elif words[0] == "!NODE":
            name = pyFistr.getValName(header, "NGRP")
            if name != "":
                data = headerData[1]
                data = list(map(lambda x: x[0], data))
                nodeGrps.append([name, data])
    #grp名をチェック。
    nodeGrps = checkSameGroupName(nodeGrps)
    #vtkを作成
    for nodeGrp in nodeGrps:
        #nodeGrpNameDictを作成
        createNodeGrpNameDict(nodeGrp)
        # #nodeGrp毎にvtkを作成
        # name = nodeGrp[0]
        # data = nodeGrp[1]
        # vtkLines = getVtkNodeData(data, nodesDict)
        # #file保存
        # fileName = currDir + os.sep + SaveFolder + os.sep + "nodeGrp_" + name + ".vtu"
        # f = open(fileName, "w"); f.writelines(vtkLines); f.close()
    return nodeGrps

#
#  createNodeGrpNameDict
def createNodeGrpNameDict(nodeGrp):
    """ nodeGrpNameDictを作成"""
    global nodeGrpNameDict
    ngrp = nodeGrp[0]
    data = nodeGrp[1]
    for nodeNo in data:
        if nodeNo in nodeGrpNameDict.keys():
            nodeGrpNameDict[nodeNo].append(ngrp)
        else:    
            nodeGrpNameDict[nodeNo] = [ngrp]

#
#  createSurfaceGrpVtkFiles
#----------------------------
def createSurfaceGrpVtkFiles(meshHeaderData, nodesDict, elementDict):
    """ 表面group名毎にvtkFileを作成する。"""
    #surfaceGroupを作成
    surfaceGrps = []
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!SGROUP":
            name = pyFistr.getValName(header, "SGRP")
            data = headerData[1]
            surfaceGrps.append([name, data])
    #grp名をチェック。
    surfaceGrps = checkSameGroupName(surfaceGrps)
    #vtkを作成
    for surfaceGrp in surfaceGrps:
        #surface毎にvtkを作成
        name = surfaceGrp[0]
        data = surfaceGrp[1]
        surData = getSurfaceDataFromFaceNo(name, data, elementDict)
        # vtkLines = getVtkSurfaceData(surData, nodesDict)
        # #file保存
        # fileName = currDir + os.sep + SaveFolder + os.sep + "surfaceGrp_" + name + ".vtu"
        # f = open(fileName, "w"); f.writelines(vtkLines); f.close()
    return surfaceGrps

#
#  remakeFaceNodesOrder
def remakeFaceNodesOrder(faceNodes):
    """ 2次要素の場合、連番のnode順をfistrの2次要素の順の置き換える"""
    faceOrderDict = {
        3: [[1,2,3], cm.order_3],
        4: [[1,2,3,4], cm.order_4],
        6: [[1,3,5,4,6,2], cm.order_6],
        8: [[1,3,5,7,2,4,6,8], cm.order_8]
        }
    faceOrder = faceOrderDict[len(faceNodes)][0]
    func = faceOrderDict[len(faceNodes)][1]
    newFaceNodes = func(faceNodes, faceOrder)
    return newFaceNodes

#
#  getSurfaceDataFromFaceNo
def getSurfaceDataFromFaceNo(name, data, elementDict):
    """ 要素のfaceNoから節点No列を取得する。surElmDictを作成する。"""
    global surElmDict
    faceTypeDict = {
      #node数 要素type
        3:  "231",
        4:  "241",
        6:  "232",
        8:  "242"
        }

    surData = []
    for i in range(0, len(data), 2):
        (elNo, faceNo) = data[i:i+2]
        elCont = elementDict[elNo]
        elNodes = elCont[0]
        elType = elCont[1]
        faceNodes0 = cm.fistr_elFaceNode(elType, elNodes, faceNo)
        #faceNodesを正しく修正する（2次要素の場合、要修正）
        faceNodes = remakeFaceNodesOrder(faceNodes0)
        #surElmDictを作成
        keyNodes = faceNodes[:]
        keyNodes.sort()
        key = tuple(keyNodes)
        if key in surElmDict.keys():
            surElmDict[key]["SGRP"][name] = 1
        else:
            grpDict = {"EGRP":{}, "SGRP":{}}
            grpDict["SGRP"][name] = 1
            surElmDict[key] = grpDict
        #vtkに変換
        faceType = faceTypeDict[len(faceNodes)]
        (vtkName, vtkNodes) = cm.fistr2vtk_el(faceType, faceNodes)
        surData.append([vtkNodes, vtkName])
    return surData

#
#  checkSameGroupName
#---------------------
def checkSameGroupName(Grps):
    """ 同じgroup名があるかチェック。あれば、まとめる。"""
    newGrps = []
    for grp in Grps:
        name = grp[0]
        data = grp[1]
        flag = 0
        for newGrp in newGrps:
            if newGrp[0] == name:
                flag = 1
                #同じgrpNameあり
                newGrp[1] += data
                break
        if flag == 0:
            newGrps.append(grp)
    return newGrps

#
#  createElementGrpVtkFiles
#--------------------------
def createElementGrpVtkFiles(meshHeaderData, nodesDict, elementDict, surfaceGrps, nodeGrps):
    """ 要素group毎にvtkFileを作成する"""
    #elementGroupを作成
    elementGrps = []
    #!ELEMENTを作成
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!ELEMENT":
            name = pyFistr.getValName(header, "EGRP")
            if name == "":
                name = "--undefine--"
            data = headerData[1]
            elementGrps.append([name, data])
    #同じgrpNameをチェック、あればまとめる
    elementGrps = checkSameGroupName(elementGrps)
    #!EGROUPを取得
    for headerData in meshHeaderData:
        header = headerData[0]
        words = pyFistr.deleteSp(header).split(",")
        if words[0] == "!EGROUP":
            name = pyFistr.getValName(header, "EGRP")
            data = []
            for elNo in headerData[1]:
                nodes = elementDict[elNo][0]
                elm = [elNo] + nodes
                data.append(elm)
            elementGrps.append([name, data])

    #solid要素を表面要素に変換、surElmDictを作成
    #  予め、surElmDictを完成させる
    faceElmData = []
    for elementGrp in elementGrps:
        #element毎にvtkを作成
        name = elementGrp[0]
        data = elementGrp[1]
        #solidをsurfaceに変換, surElmDictを作成
        data, elementDict, elmFaceNos = changeSolidToFaceElement(name, data, elementDict)
        #変換データを保存
        faceElmData.append([data, elementDict, elmFaceNos])    
    #surElmDictを使ってvtkDataを作成
    for i in range(len(elementGrps)):
        elementGrp = elementGrps[i]
        #element毎にvtkを作成
        name = elementGrp[0]
        #solidのsurfaceDataを取得
        [data, elementDict, elmFaceNos] = faceElmData[i]
        #vtkDataを作成
        elmData = getElementDataForVtk(data, elementDict)
        elmNodeDict, allNodes = getElmNodeDictForVtk(elmData)
        vtkLines = getVtkElementData(name, elmNodeDict, allNodes, elmData, nodesDict, elementGrps, surfaceGrps, nodeGrps, elmFaceNos)
        fileName = currDir + os.sep + SaveFolder + os.sep + "elementGrp_" + name + ".vtu"
        f = open(fileName, "w"); f.writelines(vtkLines); f.close()
        #edgeを作成
        vtkLines = getVtkEdgeData(elmNodeDict, allNodes, elmData, nodesDict)
        fileName = currDir + os.sep + SaveFolder + os.sep + "edgeElementGrp_" + name + ".vtu"
        f = open(fileName, "w"); f.writelines(vtkLines); f.close()

#
#  getVtkEdgeData
def getVtkEdgeData(elmNodeDict, allNodes, elmData, nodesDict):
    """ edgeのVtkを作成する。"""
    vtkEdgesDict = {
        "3":[[0,1]],                #111, beam
        "5":[[0,1],[1,2],[2,0]],    #231, 三角形
        "9":[[0,1],[1,2],[2,3],[3,0]],  #241, 四角形
        "10":[[0,1],[1,2],[2,0],[0,3],[1,3],[2,3]],     #341, tetra  
        "12":[[0,1],[1,2],[2,3],[3,0],[4,5],[5,6],[6,7],[7,4],[0,4],[1,5],[2,6],[3,7]],  #361, hexa
        "13":[[0,1],[1,2],[2,0],[3,4],[4,5],[5,3],[0,3],[1,4],[2,5]],                   #351, prism
        "21":[[0,2,1]],             #112, beam2次
        "22":[[0,3,1],[1,4,2],[2,5,0]],     #232, 三角形2次
        "23":[[0,4,1],[1,5,2],[2,6,3],[3,7,0]], #242, 四角形1次
        "24":[[0,4,1],[1,5,2],[2,6,0],[0,7,3],[1,8,3],[2,9,3]], #342, tetra2次
        "25":[[0,8,1],[1,9,2],[2,10,3],[3,11,0],[4,12,5],[5,13,6],[6,14,7],[7,15,4],[0,16,4],[1,17,5],[2,18,6],[3,19,7]],    #362, hexa2次
        "26":[[0,6,1],[1,7,2],[2,8,0],[3,9,4],[4,10,5],[5,11,3],[0,12,3],[1,13,4],[2,14,5]],        #352, prism2次
        }
    #edgeElmDictを作成
    #  edgeElmDict = {edge:vtkElmNos}
    edgeData = []
    for nodes, elmType in elmData:
        vtkEdgeNos = vtkEdgesDict[elmType]
        edges = []
        for vtkEdge in vtkEdgeNos:
            edge = list(map(lambda x: nodes[x], vtkEdge))
            edge.sort()
            edges.append(tuple(edge))
        edgeData.append(edges)
    edgeElmDict = {}
    for elmNo in range(len(edgeData)):
        edges = edgeData[elmNo]
        for edge in edges:
            if edge in edgeElmDict.keys():
                edgeElmDict[edge].append(elmNo)
            else:
                edgeElmDict[edge] = [elmNo]

    edgeKeys = list(edgeElmDict.keys())
    edgeKeys.sort()

    lines  = ['<?xml version="1.0"?>\n']
    lines += ['<VTKFile type="UnstructuredGrid" version="1.0">\n']
    lines += ['<UnstructuredGrid>\n']
    nPoints = len(allNodes)
    nCells = len(edgeKeys)
    lines += ['<Piece NumberOfPoints="' + str(nPoints) + '" NumberOfCells="' + str(nCells) + '">\n']
    lines += ['<Points>\n']
    #座標データ
    lines += ['<DataArray type="Float32" NumberOfComponents="3" format="ascii">\n']
    for nodeNo in allNodes:
        loc = nodesDict[nodeNo]
        #lines += [" ".join(map(str, loc)) + "\n"]
        #座標は、13.6eで出力する
        lines += [" ".join(map(lambda x: "%13.6e" % x, loc)) + "\n"]
    lines += ['</DataArray>\n']
    lines += ['</Points>\n']

    lines += ['<Cells>\n']
    #connectivity
    lines += ['<DataArray type="Int32" Name="connectivity" format="ascii">\n']
    for elmNodes in edgeKeys:
        newNodes = list(map(lambda x: elmNodeDict[x], elmNodes))
        lines += [" ".join(map(str, newNodes)) + "\n"]
    lines += ['</DataArray>\n']
    #offsets
    lines += ['<DataArray type="Int32" Name="offsets" format="ascii">\n']
    n = 0
    for elmNodes in edgeKeys:
        n += len(elmNodes)
        lines += [str(n) + "\n"]
    lines += ['</DataArray>\n']
    #types
    lines += ['<DataArray type="UInt8" Name="types" format="ascii">\n']
    for elmNodes in edgeKeys:
        if len(elmNodes) == 2:
            vtkName = "3"
        else:
            vtkName = "21"
        lines += [vtkName + "\n"]
    lines += ['</DataArray>\n']
    lines += ['</Cells>\n']

    lines += ['<CellData>\n']
    #pairElmNo
    lines += ['<DataArray type="Int32" Name="pairElmNo" NumberOfComponents="2" format="ascii">\n']
    line = []
    for elmNodes in edgeKeys:
        elmNos = edgeElmDict[elmNodes]
        if len(elmNos) == 1:
            vals = [str(elmNos[0]), "-1"]
        else:
            vals = [str(elmNos[0]), str(elmNos[1])]
        line += vals
    lines += [" ".join(line) + "\n"]
    lines += ['</DataArray>\n']
    lines += ['</CellData>\n']
    lines += ['</Piece>\n']
    lines += ['</UnstructuredGrid>\n']
    lines += ['</VTKFile>\n']
    return lines

#
#  getElmNodeDictForVtk
def getElmNodeDictForVtk(elmData):
    """ elmDataのnodeNoをvtkのNodeNoに変換する辞書を作成する"""
    elmNodeDict = {}
    for nodes, _name in elmData:
        for node in nodes:
            elmNodeDict[node] = 0
    allNodes = list(elmNodeDict.keys())
    allNodes.sort()
    for i in range(len(allNodes)):
        nodeNo = allNodes[i]
        elmNodeDict[nodeNo] = i     #newNodeNoを設定
    return elmNodeDict, allNodes

#
#  changeSolidToFaceElement
def changeSolidToFaceElement(grpName, elmData, elementDict):
    """ solid要素をface要素に変換する。shellやbeamはそのまま。
    全要素（solid, shell, beam)について、surElmDictを作成する。"""
    global surElmDict
    solidElmDict = {
        "231":"face",    #三角形1次
        "232":"face",    #三角形2次
        "241":"face",    #四角形1次
        "242":"face",    #四角形2次
        "341":"solid",   #四面体1次
        "342":"solid",   #四面体2次
        "361":"solid",   #六面体1次
        "362":"solid",   #六面体2次
        "351":"solid",   #五面体1次
        "352":"solid",   #五面体2次
        "611":"beam",    #beam要素
        "641":"beam",    #beam要素（3自由度）
        "731":"shell",   #シェル三角形1次
        "732":"shell",   #シェル三角形2次
        "741":"shell",   #シェル四角形1次
        "761":"shell",   #シェル三角形1次（3自由度）
        "781":"shell"    #シェル四角形1次（3自由度）
        }
    nodesToFaceDict = {
        #node数  type    node順
        3:      ["231", [1,2,3]],
        4:      ["241", [1,2,3,4]],
        6:      ["232", [1,3,5,4,6,2]],
        8:      ["242", [1,3,5,7,2,4,6,8]]
        }
    #solidElmを取得する
    solidElmData = []
    noSolidElmData = []
    for elm in elmData:
        elmNo = elm[0]
        nodes = elm[1:]
        elmType = elementDict[elmNo][1]
        if solidElmDict[elmType] == "solid":
            solidElmData.append(elm)
        else:
            noSolidElmData.append(elm)

    #solidElmをfaceElmに変換
    faceDict = {}
    for elm in solidElmData:
        elmNo = elm[0]
        nodes = elm[1:]
        elmType = elementDict[elmNo][1]
        elmFaceNodes = cm.faceNodeOfElementDict[elmType]
        for i in range(1, len(elmFaceNodes)):
            order = elmFaceNodes[i]
            nodeNos = list(map(lambda x: nodes[x-1], order))
            nodeNos.sort()
            key = tuple(nodeNos)
            if key in faceDict.keys():
                #辞書からkeyを削除(既に定義済みのface)
                _dummy = faceDict.pop(key)
            else:
                #辞書に追加
                faceDict[key] = [elmNo, i]

    #表面節点を取得
    elmFaceNos = []
    surfaceData = []
    elementNo = max(elementDict.keys()) + 1
    for key in faceDict.keys():
        if key in surElmDict.keys():
            surElmDict[key]["EGRP"][grpName] = 1
        else:
            grpDict = {"EGRP":{}, "SGRP":{}}
            grpDict["EGRP"][grpName] = 1
            surElmDict[key] = grpDict
        elmNo = faceDict[key][0]
        elNodes = elementDict[elmNo][0]
        elmType = elementDict[elmNo][1]
        faceNo = faceDict[key][1]
        elmFaceNos.append([elmNo, faceNo])
        order = cm.faceNodeOfElementDict[elmType][faceNo]
        nodes = list(map(lambda x: elNodes[x-1], order))
        faceType, surOrder = nodesToFaceDict[len(nodes)]
        nodes = list(map(lambda x: nodes[x-1], surOrder))
        surfaceData.append([elementNo] + nodes)
        elementDict[elementNo] = [nodes, faceType]
        elementNo += 1
    #solid以外のelementのsurElmDictを作成
    for elm in noSolidElmData:
        elmNo = elm[0]
        nodes = elm[1:]
        #elmFaceNos.append([0, 0])
        #elmFaceNos.append([elmNo, 0])
        elmType = elementDict[elmNo][1]
        if solidElmDict[elmType] == "shell":
            faceNo = 0      #shell
        else:
            faceNo = 99     #beam
        elmFaceNos.append([elmNo, faceNo])
        nodes.sort()
        key = tuple(nodes)
        if key in surElmDict.keys():
            surElmDict[key]["EGRP"][grpName] = 1
        else:
            grpDict = {"EGRP":{}, "SGRP":{}}
            grpDict["EGRP"][grpName] = 1
            surElmDict[key] = grpDict
    elmData = surfaceData + noSolidElmData
    return elmData, elementDict, elmFaceNos

#
#  getElementDataForVtk
def getElementDataForVtk(data, elementDict):
    """ fistr形式の要素データをvtk形式の要素データに変換する。"""
    elmData = []
    for i in range(len(data)):
        elNo = data[i][0]
        nodes = data[i][1:]
        elCont = elementDict[elNo]
        elType = elCont[1]
        (vtkName, vtkNodes) = cm.fistr2vtk_el(elType, nodes)
        elmData.append([vtkNodes, vtkName])
    return elmData

#
#  getVtkElementData
def getVtkElementData(elmGrpName, elmNodeDict, allNodes, elmData, nodesDict, elementGrps, surfaceGrps, nodeGrps, elmFaceNos):
    """ 要素group毎のvtkファイルを作成する。"""
    lines  = ['<?xml version="1.0"?>\n']
    lines += ['<VTKFile type="UnstructuredGrid" version="1.0">\n']
    lines += ['<UnstructuredGrid>\n']
    nPoints = len(allNodes)
    nCells = len(elmData)
    lines += ['<Piece NumberOfPoints="' + str(nPoints) + '" NumberOfCells="' + str(nCells) + '">\n']
    lines += ['<Points>\n']
    #座標データ
    lines += ['<DataArray type="Float32" NumberOfComponents="3" format="ascii">\n']
    for nodeNo in allNodes:
        loc = nodesDict[nodeNo]
        #lines += [" ".join(map(str, loc)) + "\n"]
        #座標は、13.6eで出力する
        lines += [" ".join(map(lambda x: "%13.6e" % x, loc)) + "\n"]
    lines += ['</DataArray>\n']
    lines += ['</Points>\n']
    lines += ['<Cells>\n']
    #connectivity
    lines += ['<DataArray type="Int32" Name="connectivity" format="ascii">\n']
    for elmNodes, vtkName in elmData:
        newNodes = list(map(lambda x: elmNodeDict[x], elmNodes))
        lines += [" ".join(map(str, newNodes)) + "\n"]
    lines += ['</DataArray>\n']
    #offsets
    lines += ['<DataArray type="Int32" Name="offsets" format="ascii">\n']
    n = 0
    for elmNodes, vtkName in elmData:
        n += len(elmNodes)
        lines += [str(n) + "\n"]
    lines += ['</DataArray>\n']
    #types
    lines += ['<DataArray type="UInt8" Name="types" format="ascii">\n']
    for elmNodes, vtkName in elmData:
        lines += [vtkName + "\n"]
    lines += ['</DataArray>\n']
    lines += ['</Cells>\n']
    #grpName
    lines += ['<CellData>\n']
    #  elmNo, faceNo
    if len(elmFaceNos) > 0:
        #elmNo
        lines += ['<DataArray type="Int32" Name="elmNo" NumberOfComponents="1" format="ascii">\n']
        line = []
        for elmFaceNo in elmFaceNos:
            elmNo = elmFaceNo[0]
            line.append(str(elmNo))
        lines += [" ".join(line) + "\n"]
        lines += ['</DataArray>\n']
        #faceNo
        lines += ['<DataArray type="UInt8" Name="faceNo" NumberOfComponents="1" format="ascii">\n']
        line = []
        for elmFaceNo in elmFaceNos:
            faceNo = elmFaceNo[1]
            line.append(str(faceNo))
        lines += [" ".join(line) + "\n"]
        lines += ['</DataArray>\n']
    #  elementGrp
    for egrpName, _data in elementGrps:
        lines += ['<DataArray type="UInt8" Name="EGRP_' + egrpName
                + '" NumberOfComponents="1" format="ascii">\n']
        line = []
        for elmNodes, vtkName in elmData:
            nodes = elmNodes[:]
            nodes.sort()
            key = tuple(nodes)
            if key in surElmDict.keys():
                if egrpName in surElmDict[key]["EGRP"]:
                    line.append("1")
                else:
                    line.append("0")
            else:
                line.append("0")
        lines += [" ".join(line) + "\n"]
        lines += ['</DataArray>\n']
    #  surfaceGrp
    for sgrpName, _data in surfaceGrps:
        lines += ['<DataArray type="UInt8" Name="SGRP_' + sgrpName
                + '" NumberOfComponents="1" format="ascii">\n']
        line = []
        for elmNodes, vtkName in elmData:
            nodes = elmNodes[:]
            nodes.sort()
            key = tuple(nodes)
            if key in surElmDict.keys():
                if sgrpName in surElmDict[key]["SGRP"]:
                    line.append("1")
                else:
                    line.append("0")
            else:
                line.append("0")
        lines += [" ".join(line) + "\n"]
        lines += ['</DataArray>\n']
    lines += ['</CellData>\n']
    #  nodeGrp
    lines += ['<PointData>\n']
    lines += ['<DataArray type="Int32" Name="nodeNo" NumberOfComponents="1" '
                + 'format="ascii">\n']
    line = []
    for nodeNo in allNodes:
        line.append(str(nodeNo))
    lines += [" ".join(line) + "\n"]
    lines += ['</DataArray>\n']
    for ngrpName, _data in nodeGrps:
        lines += ['<DataArray type="UInt8" Name="NGRP_' + ngrpName
                + '" NumberOfComponents="1" format="ascii">\n']
        line = []
        for key in allNodes:
            if key in nodeGrpNameDict.keys():
                #if nodeGrpNameDict[key] == ngrpName:
                if ngrpName in nodeGrpNameDict[key]:
                    line.append("1")
                else:
                    line.append("0")
            else:
                line.append("0")
        lines += [" ".join(line) + "\n"]
        lines += ['</DataArray>\n']
    lines += ['</PointData>\n']
    lines += ['</Piece>\n']
    lines += ['</UnstructuredGrid>\n']
    lines += ['</VTKFile>\n']
    return lines

#
#  saveVersion
#---------------
def saveVersion():
    """ vtkFolder内にversionNoを保存"""
    versionFile = currDir + os.sep + SaveFolder + os.sep + "version_" + version
    cont = "version " + version
    f = open(versionFile, "w"); f.write(cont); f.close()

#-----------------
#  createVtkFiles
#-----------------
def createVtkFiles(tempDir):
    """ 要素、表面、節点group毎にvtkFileを作成する。"""
    global currDir
    currDir = tempDir
    os.chdir(currDir)
    #mesh読み込み
    meshHeaderData = getMeshHeaderData()
    nodesDict = getNodesDict(meshHeaderData)
    elementDict = getElementDict(meshHeaderData)
    clearSaveFolder()
    nodeGrps = createNodeGrpVtkFiles(meshHeaderData, nodesDict)
    surfaceGrps = createSurfaceGrpVtkFiles(meshHeaderData, nodesDict, elementDict)
    createElementGrpVtkFiles(meshHeaderData, nodesDict, elementDict, surfaceGrps, nodeGrps)
    saveVersion()

#-----------------
#  renameNodeGrp
#-----------------
def renameNodeGrp(vtkDir, ngrpName, newName):
    """ 全egrpファイル内のngrpNameをnewNameに変更する"""
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        lines = readFileLines(fileName)
        si = lines.index("<PointData>\n")
        if si < 0:
            for i in range(len(lines)):
                if lines[i][:len("<PointData>")] == "<PointData>":
                    si = i
                    break
        if si >= 0:
            for i in range(si, len(lines)):
                line = lines[i]
                if line.find("<DataArray") >= 0:
                    words = line.split()
                    flag = 0
                    for ii in range(len(words)):
                        if words[ii][:len("Name=")] == "Name=":
                            if words[ii].split("=")[1] == '"NGRP_' + ngrpName + '"':
                                words[ii] = 'Name="NGRP_' + newName + '"'
                                newLine = " ".join(words) + "\n"
                                lines[i] = newLine
                                flag = 1
                                break
                    if flag == 1:
                        break
            writeFileLines(fileName, lines)

#--------------------
#  renameSurfaceGrp
#--------------------
def renameSurfaceGrp(vtkDir, sgrpName, newName):
    """ 全egrpファイル内のsgrpNameをnewNameに変更する"""
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        lines = readFileLines(fileName)
        si = lines.index("<CellData>\n")
        if si < 0:
            for i in range(len(lines)):
                if lines[i][:len("<CellData>")] == "<CellData>":
                    si = i
                    break
        if si >= 0:
            for i in range(si, len(lines)):
                line = lines[i]
                if line.find("<DataArray") >= 0:
                    words = line.split()
                    flag = 0
                    for ii in range(len(words)):
                        if words[ii][:len("Name=")] == "Name=":
                            if words[ii].split("=")[1] == '"SGRP_' + sgrpName + '"':
                                words[ii] = 'Name="SGRP_' + newName + '"'
                                newLine = " ".join(words) + "\n"
                                lines[i] = newLine
                                flag = 1
                                break
                    if flag == 1:
                        break
            writeFileLines(fileName, lines)

#-------------------
#  renameElementGrp
#-------------------
def renameElementGrp(vtkDir, egrpName, newName):
    """ 全egrpファイル内のegrpNameをnewNameに変更する"""
    #vtkFile名を変更する
    orgFile = vtkDir + os.sep + "elementGrp_" + egrpName + ".vtu"
    newFile = vtkDir + os.sep + "elementGrp_" + newName + ".vtu"
    orgEdgeFile = vtkDir + os.sep + "edgeElementGrp_" + egrpName + ".vtu"
    newEdgeFile = vtkDir + os.sep + "edgeElementGrp_" + newName + ".vtu"
    os.rename(orgFile, newFile)
    os.rename(orgEdgeFile, newEdgeFile)
    #  renameが完了するまで待つ
    while os.path.exists(newFile) == False and os.path.exists(newEdgeFile):
        time.sleep(0.1)
    #vtkFileの内容を修正
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        lines = readFileLines(fileName)
        si = lines.index("<CellData>\n")
        if si < 0:
            for i in range(len(lines)):
                if lines[i][:len("<CellData>")] == "<CellData>":
                    si = i
                    break
        if si >= 0:
            for i in range(si, len(lines)):
                line = lines[i]
                if line.find("<DataArray") >= 0:
                    words = line.split()
                    flag = 0
                    for ii in range(len(words)):
                        if words[ii][:len("Name=")] == "Name=":
                            if words[ii].split("=")[1] == '"EGRP_' + egrpName + '"':
                                words[ii] = 'Name="EGRP_' + newName + '"'
                                newLine = " ".join(words) + "\n"
                                lines[i] = newLine
                                flag = 1
                                break
                    if flag == 1:
                        break
            writeFileLines(fileName, lines)

#----------------
#  deleteNodeGrp
#----------------
def deleteNodeGrp(vtkDir, grpNames):
    """ 全egrpファイル内のngrpNamesを削除する"""
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        lines = readFileLines(fileName)
        si = lines.index("<PointData>\n")
        if si < 0:
            for i in range(len(lines)):
                if lines[i][:len("<PointData>")] == "<PointData>":
                    si = i
                    break
        if si >= 0:
            names = list(map(lambda x: 'Name="NGRP_'+x+'"', grpNames))
            newLines = lines[:si]
            i = si
            while i < len(lines):
                line = lines[i]
                if line.find("<DataArray") >= 0:
                    words = line.split()
                    flag = 0
                    for ii in range(len(words)):
                        if words[ii] in names:
                            i += 2      #2行分(data, footer)skipする
                            flag = 1
                            break
                    if flag == 0:
                        newLines.append(line)
                else:
                    newLines.append(line)
                i += 1
            #newLinesを保存               
            writeFileLines(fileName, newLines)

#-------------------
#  deleteSurfaceGrp
#-------------------
def deleteSurfaceGrp(vtkDir, grpNames):
    """ 全egrpファイル内のsgrpNamesを削除する"""
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        lines = readFileLines(fileName)
        si = lines.index("<CellData>\n")
        if si < 0:
            for i in range(len(lines)):
                if lines[i][:len("<CellData>")] == "<CellData>":
                    si = i
                    break
        if si >= 0:
            names = list(map(lambda x: 'Name="SGRP_'+x+'"', grpNames))
            newLines = lines[:si]
            i = si
            while i < len(lines):
                line = lines[i]
                if line.find("<DataArray") >= 0:
                    words = line.split()
                    flag = 0
                    for ii in range(len(words)):
                        if words[ii] in names:
                            i += 2      #2行分(data, footer)skipする
                            flag = 1
                            break
                    if flag == 0:
                        newLines.append(line)
                else:
                    newLines.append(line)
                i += 1
            #newLinesを保存               
            writeFileLines(fileName, newLines)

#-------------------
#  deleteElementGrp
#-------------------
def deleteElementGrp(vtkDir, grpNames):
    """ 全egrpファイル内のsgrpNamesを削除する"""
    #対象のvtkFileを削除
    delFiles = list(map(lambda x: vtkDir+os.sep+"elementGrp_"+x+".vtu", grpNames))
    delEdgeFiles = list(map(lambda x: vtkDir+os.sep+"edgeElementGrp_"+x+".vtu", grpNames))
    for delFile in delFiles + delEdgeFiles:
        os.remove(delFile)
    #vtkFileの内容を修正
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        #削除対象以外のvtkFile？
        if not fileName in delFiles:
            #file毎に内容を修正        
            lines = readFileLines(fileName)
            si = lines.index("<CellData>\n")
            if si < 0:
                for i in range(len(lines)):
                    if lines[i][:len("<CellData>")] == "<CellData>":
                        si = i
                        break
            if si >= 0:
                names = list(map(lambda x: 'Name="EGRP_'+x+'"', grpNames))
                newLines = lines[:si]
                i = si
                while i < len(lines):
                    line = lines[i]
                    if line.find("<DataArray") >= 0:
                        words = line.split()
                        flag = 0
                        for ii in range(len(words)):
                            if words[ii] in names:
                                i += 2      #2行分(data, footer)skipする
                                flag = 1
                                break
                        if flag == 0:
                            newLines.append(line)
                    else:
                        newLines.append(line)
                    i += 1
                #newLinesを保存               
                writeFileLines(fileName, newLines)

#-------------------
#  addDeleteNodeGrps
#-------------------
def addDeleteNodeGrps(vtkDir, grpNames, newName):
    """ 全egrpファイル内に対し、newNameにgrpNamesを加え、
    grpNamesを削除する"""
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        lines = readFileLines(fileName)
        si = lines.index("<PointData>\n")
        if si < 0:
            #見つからなかった場合、1行ずつ確認する。
            for i in range(len(lines)):
                if lines[i][:len("<PointData>")] == "<PointData>":
                    si = i
                    break
        if si >= 0:
            newLines = lines[:si]
            #grpNamesの値を取得する。
            grpNamesData = []
            repLines = []
            flag = 0
            for i in range(si, len(lines)):
                line = lines[i]
                if line.find("<DataArray") >= 0:
                    for grpName in grpNames:
                        if line.find('Name="NGRP_' + grpName + '"') >= 0:
                            grpData = list(map(int, lines[i+1].split()))
                            grpNamesData.append(grpData)
                            repLines.append(i)
                            flag += 1
                            break
                if flag >= len(grpNames):
                    break
            #newNameの値を取得
            for i in range(si, len(lines)):
                line = lines[i]
                if line.find("<DataArray") >= 0:
                    if line.find('Name="NGRP_' + newName + '"') >= 0:
                        newNameData = list(map(int, lines[i+1].split()))
                        grpNamesData.append(newNameData)
                        repLines.append(i)
            #grpNames, newNameの値を加算する
            nData = len(grpNamesData[0])
            res = [0 for i in range(nData)]
            for data in grpNamesData:
                res = list(map(lambda x, y: x + y, res, data))
            for i in range(nData):
                if res[i] > 0:
                    res[i] = 1
            #newNameLineを置き換え
            rep_i = repLines[-1]
            header = lines[rep_i]
            words = header.split()
            for i in range(len(words)):
                if words[i][:len("Name=")] == "Name=":
                    words[i] = 'Name="NGRP_' + newName + '"'
                    break
            header = " ".join(words) + "\n"
            lines[rep_i] = header
            newNameLine = " ".join(list(map(str, res))) + "\n"
            lines[rep_i+1] = newNameLine
            repLines = repLines[:-1]
            #grpNameを削除
            newLines = []
            i = si
            while i < len(lines):
                #repLinesから2行を削除
                if i in repLines:
                    i += 2
                else:
                    newLines.append(lines[i])
                i += 1
            newLines = lines[:si] + newLines
            #newLinesを保存
            writeFileLines(fileName, newLines)

#----------------------
#  addDeleteSurfaceGrps
#----------------------
def addDeleteSurfaceGrps(vtkDir, grpNames, newName):
    """ 全egrpファイル内に対し、newNameにgrpNamesを加え、
    grpNameを削除する"""
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        lines = readFileLines(fileName)
        si = lines.index("<CellData>\n")
        if si < 0:
            #見つからなかった場合、1行づつ確認する。
            for i in range(len(lines)):
                if lines[i][:len("<CellData>")] == "<CellData>":
                    si = i
                    break
        if si >= 0:
            newLines = lines[:si]
            #grpNamesの値を取得する
            grpNamesData = []
            repLines = []
            flag = 0
            for i in range(si, len(lines)):
                line = lines[i]
                if line.find("<DataArray") >= 0:
                    for grpName in grpNames:
                        if line.find('Name="SGRP_' + grpName + '"') >= 0:
                            grpData = list(map(int, lines[i+1].split()))
                            grpNamesData.append(grpData)
                            repLines.append(i)
                            flag += 1
                            break
                if flag >= len(grpNames):
                    break
            #newNameの値を取得
            for i in range(si, len(lines)):
                line = lines[i]
                if line.find("<DataArray") >= 0:
                    if line.find('Name="SGRP_' + newName + '"') >= 0:
                        newNameData = list(map(int, lines[i+1].split()))
                        grpNamesData.append(newNameData)
                        repLines.append(i)
            #grpNames, newNameの値を加算する
            nData = len(grpNamesData[0])
            res = [0 for i in range(nData)]
            for data in grpNamesData:
                res = list(map(lambda x, y: x + y, res, data))
            for i in range(nData):
                if res[i] > 0:
                    res[i] = 1
            #newNameLineを置き換え
            rep_i = repLines[-1]
            header = lines[rep_i]
            words = header.split()
            for i in range(len(words)):
                if words[i][:len("Name=")] == "Name=":
                    words[i] = 'Name="SGRP_' + newName + '"'
                    break
            header = " ".join(words) + "\n"
            lines[rep_i] = header
            newNameLine = " ".join(list(map(str, res))) + "\n"
            lines[rep_i+1] = newNameLine
            repLines = repLines[:-1]
            #grpNameを削除
            newLines = []
            i = si
            while i < len(lines):
                #repLinesから2行を削除
                if i in repLines:
                    i += 2
                else:
                    newLines.append(lines[i])
                i += 1
            newLines = lines[:si] + newLines
            #newLinesを保存
            writeFileLines(fileName, newLines)

#----------------------
#  addDeleteSurfaceGrp
#----------------------
def addDeleteSurfaceGrp(vtkDir, grpName, newName):
    """ 全egrpファイル内に対し、newNameにgrpNameを加え、
    grpNameを削除する"""
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        lines = readFileLines(fileName)
        si = lines.index("<CellData>\n")
        if si < 0:
            for i in range(len(lines)):
                if lines[i][:len("<CellData>")] == "<CellData>":
                    si = i
                    break
        if si >= 0:
            newLines = lines[:si]
            #grpName, newNameのindexを確認
            flag = 0
            for i in range(si, len(lines)):
                line = lines[i]
                if line.find("<DataArray") >= 0:
                    if line.find('Name="SGRP_' + grpName + '"') >= 0:
                        i_grpName = i + 1
                        flag += 1
                    elif line.find('Name="SGRP_' + newName + '"') >= 0:
                        i_newName = i + 1
                        flag += 1
                if flag >= 2:
                    break
            #newNameにgrpNameを加算
            grpNameData = list(map(int, lines[i_grpName].split()))
            newNameData = list(map(int, lines[i_newName].split()))
            for i in range(len(grpNameData)):
                if grpNameData[i] != 0:
                    newNameData[i] = 1
            newNameLine = " ".join(list(map(str, newNameData))) + "\n"
            lines[i_newName] = newNameLine
            #grpNameを削除
            newLines = lines[:i_grpName-1] + lines[i_grpName+2:]
            #newLinesを保存
            writeFileLines(fileName, newLines)

#----------------------
#  addDeleteElementGrp
#----------------------
def addDeleteElementGrp(vtkDir, grpName, newName):
    """ 全egrpファイル内に対し、newNameにgrpNameを加え、
    grpNameを削除する"""
    #対象のvtkFileを削除
    delFile = vtkDir + os.sep + "elementGrp_" + grpName + ".vtu"
    os.remove(delFile)
    #vtkFileの修正
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        #delFile以外のfile？
        if fileName != delFile:
            lines = readFileLines(fileName)
            si = lines.index("<CellData>\n")
            if si < 0:
                for i in range(len(lines)):
                    if lines[i][:len("<CellData>")] == "<CellData>":
                        si = i
                        break
            if si >= 0:
                newLines = lines[:si]
                #grpName, newNameのindexを確認
                flag = 0
                for i in range(si, len(lines)):
                    line = lines[i]
                    if line.find("<DataArray") >= 0:
                        if line.find('Name="EGRP_' + grpName + '"') >= 0:
                            i_grpName = i + 1
                            flag += 1
                        elif line.find('Name="EGRP_' + newName + '"') >= 0:
                            i_newName = i + 1
                            flag += 1
                    if flag >= 2:
                        break
                #newNameにgrpNameを加算
                grpNameData = list(map(int, lines[i_grpName].split()))
                newNameData = list(map(int, lines[i_newName].split()))
                for i in range(len(grpNameData)):
                    if grpNameData[i] != 0:
                        newNameData[i] = 1
                newNameLine = " ".join(list(map(str, newNameData))) + "\n"
                lines[i_newName] = newNameLine
                #grpNameを削除
                newLines = lines[:i_grpName-1] + lines[i_grpName+2:]
                #newLinesを保存
                writeFileLines(fileName, newLines)

#-------------------
#  addNodeGrp
#-------------------
def addNodeGrp(vtkFile, ngrpName, ngrpData):
    """ 指定したegrpファイル内にngrpを追加する。
    同名のngrpが存在する場合は、置き換える"""
    lines = readFileLines(vtkFile)
    nPoints = getNumberOfPoints(lines)
    nodeNos = getNodeNoOfFistr(lines)
    headerLine = createNodeGrpHeader(ngrpName)
    stat, si, orgData = getReplaceInsertPoint(ngrpName, lines, nPoints)
    dataLine = createNodeGrpData(nPoints, nodeNos, ngrpData, orgData)
    footerLine = createNodeGrpFooter()
    lines = insertNgrpGrpLine(headerLine, dataLine, footerLine, ngrpName, lines, stat, si)
    writeFileLines(vtkFile, lines)
    if stat == "insert":
        writeNodeGrpOtherFiles(vtkFile, ngrpName)

#--------------------
#  addSurfaceGrp
#--------------------
def addSurfaceGrp(vtkFile, sgrpName, sgrpData):
    """ 指定したegrpファイル内にsgrpを追加する。
    同名のngrpが存在する場合は、置き換える"""
    lines = readFileLines(vtkFile)
    nCells = getNumberOfCells(lines)
    cellNos, faceNos = getCellFaceNoOfFistr(lines)
    headerLine = createSurfaceGrpHeader(sgrpName)
    stat, si, orgData = getPeplaceInsertCell(sgrpName, lines, nCells)
    dataLine = createSurfaceGrpData(nCells, cellNos, faceNos, sgrpData, orgData)
    footerLine = createNodeGrpFooter()
    lines = insertSurfaceGrpLine(headerLine, dataLine, footerLine, sgrpName, lines, stat, si)
    writeFileLines(vtkFile, lines)
    if stat == "insert":
        writeSurfaceGrpOtherFiles(vtkFile, sgrpName)

#
#  writeSurfaceGrpOtherFiles
def writeSurfaceGrpOtherFiles(vtkFile, sgrpName):
    """ vtkFile以外のfileにngrpNameを追加する"""
    vtkDir = os.path.dirname(vtkFile)
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        if fileName != vtkFile:
            #sgrpを追加
            lines = readFileLines(fileName)
            nCells = getNumberOfCells(lines)
            cellData = ["0" for i in range(nCells)]
            dataLine = " ".join(cellData) + "\n"
            stat, si, _orgData = getPeplaceInsertCell(sgrpName, lines, nCells)
            if stat == "insert":
                header = createSurfaceGrpHeader(sgrpName)
                footer = createNodeGrpFooter()
                lines = lines[:si] + [header] + [dataLine] + [footer] + lines[si:]
                writeFileLines(fileName, lines)

#
#  writeNodeGrpOtherFiles
def writeNodeGrpOtherFiles(vtkFile, ngrpName):
    """ vtkFile以外のfileにngrpNameを追加する"""
    vtkDir = os.path.dirname(vtkFile)
    vtkFiles = glob.glob(vtkDir + os.sep + "elementGrp_*")
    for fileName in vtkFiles:
        if fileName != vtkFile:
            #ngrpを追加
            lines = readFileLines(fileName)
            nPoints = getNumberOfPoints(lines)
            nodeData = ["0" for i in range(nPoints)]
            dataLine = " ".join(nodeData) + "\n"
            stat, si, _orgData = getReplaceInsertPoint(ngrpName, lines, nPoints)
            if stat == "insert":
                header = createNodeGrpHeader(ngrpName)
                footer = createNodeGrpFooter()
                lines = lines[:si] + [header] + [dataLine] + [footer] + lines[si:]
                writeFileLines(fileName, lines)

#
#  getPeplaceInsertCell
def getPeplaceInsertCell(grpName, lines, nCells):
    """ 置き換え、挿入場所を取得する"""
    stat = ""
    for i in range(len(lines)):
        line = lines[i]
        if line.find("<DataArray") >= 0:
            if line.find("SGRP_"+grpName) >= 0:
                stat = "replace"
                si = i
                break
        elif line.find("</CellData>") >= 0:
            si = i
            stat = "insert"
            break
    if stat == "replace":
        orgData = list(map(int, lines[si+1].split()))
    else:
        orgData = [0 for i in range(nCells)]
    return stat, si, orgData

#
#  getReplaceInsertPoint
def getReplaceInsertPoint(grpName, lines, nPoints):
    """ 置き換え、挿入場所を確認する"""
    stat = ""
    for i in range(len(lines)):
        line = lines[i]
        if line.find("<DataArray") >= 0:
            if line.find("NGRP_"+grpName) >= 0:
                stat = "replace"
                si = i
                break
        elif line.find("</PointData>") >= 0:
            si = i
            stat = "insert"
            break
    if stat == "replace":
        orgData = list(map(int, lines[si+1].split()))
    else:
        orgData = [0 for i in range(nPoints)] 
    return stat, si, orgData

#
#  insertSurfaceGrpLine
def insertSurfaceGrpLine(header, data, footer, sgrpName, lines, stat, si):
    """ header, dataをlinesに挿入する。"""
    if stat == "insert":
        #挿入する
        lines = lines[:si] + [header] + [data] + [footer] + lines[si:]
    else:
        #置き換える（同名のngrpが存在する場合）
        lines[si] = header
        lines[si+1] = data
        lines[si+2] = footer
    return lines

#
#  createSurfaceGrpData
def createSurfaceGrpData(nCells, cellNos, faceNos, sgrpData, orgData):
    """ 「0,1」のsurfaceGrpデータを作成する"""
    cellData = orgData
    #grpDataを[cell, face]でまとめる
    grpData = []
    for i in range(0, len(sgrpData), 2):
        cellNo = sgrpData[i]
        faceNo = sgrpData[i+1]
        grpData.append([cellNo, faceNo])
    #cellNosとfaceNosを[cell, face]でまとめる
    faceCellNos = []
    for i in range(len(cellNos)):
        faceCellNos.append([cellNos[i], faceNos[i]])
    #cellDataを作成
    for faceCell in grpData:
        id = faceCellNos.index(faceCell)
        cellData[id] = 1
    cellData = list(map(str, cellData))
    line = " ".join(cellData) + "\n"
    return line

#
#  createSurfaceGrpHeader
def createSurfaceGrpHeader(sgrpName):
    """ nodeGrpのheder部を作成する"""
    line = '<DataArray type="UInt8" Name="SGRP_' + sgrpName + '" NumberOfComponents="1" format="ascii">\n'
    return line

#
#  getCellFaceNoOfFistr
def getCellFaceNoOfFistr(lines):
    """ FistrのcellNoとfaceNoを取得"""
    cellNos = []
    faceNos = []
    flag = 0
    for i in range(len(lines)):
        line = lines[i]
        if line.find('Name="elmNo"') >= 0:
            cellNos = lines[i+1].split()
            flag += 1
        elif line.find('Name="faceNo"') >= 0:
            faceNos = lines[i+1].split()
            flag += 1
        if flag == 2:
            break
    cellNos = list(map(int, cellNos))
    faceNos = list(map(int, faceNos))
    return cellNos, faceNos       

#
#  insertNgrpGrpLine
def insertNgrpGrpLine(header, data, footer, ngrpName, lines, stat, si):
    """ header, dataをlinesに挿入する。"""
    if stat == "insert":
        #挿入する（同名のngrpが存在しない場合）
        lines = lines[:si] + [header] + [data] + [footer] + lines[si:]
    else:
        #置き換える（同名のngrpが存在する場合）
        lines[si] = header
        lines[si+1] = data
        lines[si+2] = footer
    return lines

#
#  createNodeGrpFooter
def createNodeGrpFooter():
    """ nodeGrpのFooter作成"""
    return "</DataArray>\n"

#
#  createNodeGrpHeader
def createNodeGrpHeader(ngrpName):
    """ nodeGrpのheder部を作成する"""
    line = '<DataArray type="UInt8" Name="NGRP_' + ngrpName + '" NumberOfComponents="1" format="ascii">\n'
    return line

#
#  createNodeGrpData
def createNodeGrpData(nPoints, nodeNos, ngrpData, orgData):
    """ 「0,1」のnodeGrpデータを作成する"""
    pointData = orgData
    for nodeNo in ngrpData:
        id = nodeNos.index(nodeNo)
        pointData[id] = 1
    pointData = list(map(str, pointData))
    line = " ".join(pointData) + "\n"
    return line

#
#  getNodeNoOfFistr
def getNodeNoOfFistr(lines):
    """ FistrのnodeNoを取得"""
    nodeNos = []
    for i in range(len(lines)):
        line = lines[i]
        n = line.find('Name="nodeNo"')
        if n >= 0:
            nodeNos = lines[i+1].split()
            break
    nodeNos = list(map(int, nodeNos))
    return nodeNos

#
#  getNumberOfPoints
def getNumberOfPoints(lines):
    nPoints = 0
    for line in lines:
        n = line.find("NumberOfPoints")
        if n >= 0:
            word = line[n:].split()[0]
            nPoints = word.split('"')[1]
            break
    return int(nPoints)

#
#  getNumberOfCells
def getNumberOfCells(lines):
    nCells = 0
    for line in lines:
        n = line.find("NumberOfCells")
        if n >= 0:
            word = line[n:].split()[0]
            nCells = word.split('"')[1]
            break
    return int(nCells)


if __name__ == "__main__":
    import gettext
    gettext.install("app")
    _ = gettext.gettext
    if len(sys.argv) == 1:
        currDir = os.getcwd()
    else:
        currDir = sys.argv[1]
    currDir = os.path.abspath(currDir)
    createVtkFiles(currDir)
